@@ -14,6 +14,7 @@
 #if __GLASGOW_HASKELL__ >= 706
 {-# LANGUAGE PolyKinds #-}
 #endif
+#include "bifunctors-common.h"
 
 -----------------------------------------------------------------------------
 -- |
@@ -53,6 +54,10 @@ import Data.Typeable
 import GHC.Generics
 #endif
 
+#if LIFTED_FUNCTOR_CLASSES
+import Data.Functor.Classes
+#endif
+
 -- | Make a 'Bifunctor' flipping the arguments of a 'Bifunctor'.
 newtype Flip p a b = Flip { runFlip :: p b a }
   deriving ( Eq, Ord, Show, Read
@@ -64,6 +69,37 @@ newtype Flip p a b = Flip { runFlip :: p b a }
 #endif
            )
 
+#if LIFTED_FUNCTOR_CLASSES
+instance (Eq2 p, Eq a) => Eq1 (Flip p a) where
+  liftEq = liftEq2 (==)
+instance Eq2 p => Eq2 (Flip p) where
+  liftEq2 f g (Flip x) (Flip y) = liftEq2 g f x y
+
+instance (Ord2 p, Ord a) => Ord1 (Flip p a) where
+  liftCompare = liftCompare2 compare
+instance Ord2 p => Ord2 (Flip p) where
+  liftCompare2 f g (Flip x) (Flip y) = liftCompare2 g f x y
+
+instance (Read2 p, Read a) => Read1 (Flip p a) where
+  liftReadsPrec = liftReadsPrec2 readsPrec readList
+instance Read2 p => Read2 (Flip p) where
+  liftReadsPrec2 rp1 rl1 rp2 rl2 p = readParen (p > 10) $ \s0 -> do
+    ("Flip",    s1) <- lex s0
+    ("{",       s2) <- lex s1
+    ("runFlip", s3) <- lex s2
+    (x,         s4) <- liftReadsPrec2 rp2 rl2 rp1 rl1 0 s3
+    ("}",       s5) <- lex s4
+    return (Flip x, s5)
+
+instance (Show2 p, Show a) => Show1 (Flip p a) where
+  liftShowsPrec = liftShowsPrec2 showsPrec showList
+instance Show2 p => Show2 (Flip p) where
+  liftShowsPrec2 sp1 sl1 sp2 sl2 p (Flip x) = showParen (p > 10) $
+      showString "Flip {runFlip = "
+    . liftShowsPrec2 sp2 sl2 sp1 sl1 0 x
+    . showChar '}'
+#endif
+
 instance Bifunctor p => Bifunctor (Flip p) where
   first f = Flip . second f . runFlip
   {-# INLINE first #-}
